/*
********************************************************************************
*                                  uC/OS-II
*								DA
*                              ARM Cortex-M3 Port
*
* File      	: DrvDACx.C
* Version   	: V1.0
* By        	: ǿ
*
* For       	: Stm32f10x
* Mode      	: Thumb2
* Toolchain 	: 
*             		RealView Microcontroller Development Kit (MDK)
*             		Keil uVision
* Description   :  STM32F10x ڲDAC 
*					ռDAC
*					
*				
*					DACxOpen
*					DACxClose
*					DACxWrite
*					DACxRead
*					DACxIoCtl
*					DACxInstall
*					DACxNuinstall
* Date          : 2012.05.28
*******************************************************************************/



#include "DrvDACx.h"


/*******************************************************************************
* Function Name :static void DAC1Update(u16 ch1)
* Description   :DAC1ͨ
* Input         :CH1:DAֵ
* Output        :
* Other         :
* Date          :2012.05.28  14:02:17
*******************************************************************************/
static void DAC1Update(u16 ch1)
{
	/* Set DAC Channel1 DHR12L register */
	DAC_SetChannel1Data(DAC_Align_12b_R, ch1 & 0x0FFF);

	/* Start DAC Channel1 conversion by software */
	DAC_SoftwareTriggerCmd(DAC_Channel_1, ENABLE);
}

/*******************************************************************************
* Function Name :static void DAC2Update(u16 ch2)
* Description   :DAC2ͨ
* Input         :CH2:DAֵ
* Output        :
* Other         :
* Date          :2012.05.28  14:04:55
*******************************************************************************/
static void DAC2Update(u16 ch2)
{
	/* Set DAC Channel2 DHR12L register */
	DAC_SetChannel2Data(DAC_Align_12b_R, ch2 & 0x0FFF);

	/* Start DAC Channel1 conversion by software */
	DAC_SoftwareTriggerCmd(DAC_Channel_2, ENABLE);
}



/*******************************************************************************
* Function Name :static s32 DACxOpen(void *pd)
* Description   :ʹDACģ
* Input         :Ч
* Output        :
* Other         :
* Date          :2012.05.28  14:03:49
*******************************************************************************/
static s32 DACxOpen(void *pd)
{
	DAC_InitTypeDef  DAC_InitStructure;
	GPIO_InitTypeDef GPIO_InitStructure;
	
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO | RCC_APBXPeriph_GPIOX, ENABLE);
	/* DAC Periph clock enable */
	RCC_APB1PeriphClockCmd(RCC_APBXPeriph_DACx, ENABLE);
	
#if DAC_CHANNEL_1_EN
	/* Configure DAC channe1 output pin */
	GPIO_InitStructure.GPIO_Pin =  DACx_GPIOX_PIN_CH1;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_Init(DACx_GPIOX_1, &GPIO_InitStructure);
#endif

#if DAC_CHANNEL_2_EN
	/* Configure DAC channe1 output pin */
	GPIO_InitStructure.GPIO_Pin =  DACx_GPIOX_PIN_CH2;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_Init(DACx_GPIOX_2, &GPIO_InitStructure);
#endif
	
	/* DAC channel1 Configuration */
	DAC_InitStructure.DAC_Trigger = DAC_Trigger_Software;
	DAC_InitStructure.DAC_WaveGeneration = DAC_WaveGeneration_None;
	DAC_InitStructure.DAC_OutputBuffer = DAC_OutputBuffer_Disable;	//
	DAC_InitStructure.DAC_LFSRUnmask_TriangleAmplitude = DAC_TriangleAmplitude_4095;

#if DAC_CHANNEL_1_EN
	DAC_Init(DAC_Channel_1, &DAC_InitStructure);

	/* Enable DAC Channel1: Once the DAC channel1 is enabled, PA.04 is 
	automatically connected to the DAC converter. */
	DAC_Cmd(DAC_Channel_1, ENABLE);
#endif

#if DAC_CHANNEL_2_EN
	/* DAC channel2 Configuration */
	DAC_Init(DAC_Channel_2, &DAC_InitStructure);

	/* Enable DAC Channel2: Once the DAC channel2 is enabled, PA.05 is 
	automatically connected to the DAC converter. */
	DAC_Cmd(DAC_Channel_2, ENABLE);
#endif

	return (s32)DRV_NO_ERR;
}


/*******************************************************************************
* Function Name :static s32 DACxClose(void *pd)
* Description   :رDACģ
* Input         :Ч
* Output        :
* Other         :
* Date          :2012.05.28  14:08:25
*******************************************************************************/
static s32 DACxClose(void *pd)
{	
#if DAC_CHANNEL_1_EN
	DAC_Cmd(DAC_Channel_1, DISABLE);
#endif

#if DAC_CHANNEL_2_EN
	DAC_Cmd(DAC_Channel_2, DISABLE);
#endif

	return (s32)DRV_NO_ERR;
}

/*******************************************************************************
* Function Name :static s32 DACxIoCtl(u32 too, void *pd)
* Description   :DACX
* Input         :too: 	*pd
*				 1 		1ͨDAֵ	:16
*				 2 		2ͨDAֵ
* Output        :
* Other         :
* Date          :2012.05.23  14:19:51
*******************************************************************************/
static s32 DACxIoCtl(u32 too, void *pd)
{
	switch (too)
	{
#if DAC_CHANNEL_1_EN
		case 1 : DAC1Update(*(u16 *)pd);	break;
#endif

#if DAC_CHANNEL_2_EN
		case 2 : DAC2Update(*(u16 *)pd);	break;
#endif
		default : return(s32)DRV_CTRL_FAIL;
	}
	return (s32)DRV_NO_ERR;
}

/*******************************************************************************
* Function Name :s32 DACxInstall(UDFOperationsType *op)
* Description   :װDACx
* Input         :
* Output        :
* Other         :
* Date          :2012.05.13
*******************************************************************************/
s32 DACxInstall(UDFOperationsType *op)
{
	op->devOpen = DACxOpen;
	op->devClose = DACxClose;
	op->devWrite = NULL;
	op->devRead = NULL;
	op->devIoctl = DACxIoCtl;
	
	return (s32)DRV_NO_ERR;
}

/*******************************************************************************
* Function Name :s32 DACxNuinstall(UDFOperationsType *op)
* Description   :жDACx
* Input         :
* Output        :
* Other         :
* Date          :2012.05.13
*******************************************************************************/
s32 DACxNuinstall(UDFOperationsType *op)
{
	s32 res = (s32)DRV_NO_ERR;
	void *pd = NULL;
	
	if (op->devClose != NULL)
		res = op->devClose(pd);
	op->devOpen = NULL;
	op->devClose = NULL;
	op->devWrite = NULL;
	op->devRead = NULL;
	op->devIoctl = NULL;

	return res;
}






