/*
********************************************************************************
*                                  uC/OS-II
*						25LC640ⲿ洢оƬ
*                              ARM Cortex-M3 Port
*
* File      	: Drv25LC640.C
* Version   	: V1.0
* By        	: ǿ
*
* For       	: Stm32f10x
* Mode      	: Thumb2
* Toolchain 	: 
*             		RealView Microcontroller Development Kit (MDK)
*             		Keil uVision
* Description   : ⲿ洢 
*					ռGPIO 궨趨
*					
*				
*					EepromOpen
*					EepromClose
*					EepromWrite
*					EepromRead
*					EepromIoCtl
*					EepromInstall
*					EepromNuinstall
* Date          : 2012.05.28
*******************************************************************************/



#include "Drv25LC640.h"

#ifdef UCOSIII
static OS_ERR err;
#endif



 void EepromInit( void )
 {
	GPIO_InitTypeDef GpioInitdef;

	RCC_APB2PeriphClockCmd(RCC_APBXPeriph_GPIOX, ENABLE);  
	
	GpioInitdef.GPIO_Pin = GPIOX_PIN_STE;
	GpioInitdef.GPIO_Speed = GPIO_Speed_10MHz;
	GpioInitdef.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_Init(GPIOX_STE, &GpioInitdef);

	GpioInitdef.GPIO_Pin = GPIOX_PIN_SOMI;
	GpioInitdef.GPIO_Speed = GPIO_Speed_10MHz;
	GpioInitdef.GPIO_Mode = GPIO_Mode_IPD;
	GPIO_Init(GPIOX_SOMI, &GpioInitdef);

	GpioInitdef.GPIO_Pin = GPIOX_PIN_WE;
	GpioInitdef.GPIO_Speed = GPIO_Speed_10MHz;
	GpioInitdef.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_Init(GPIOX_WE, &GpioInitdef);

	GpioInitdef.GPIO_Pin = GPIOX_PIN_CLK;
	GpioInitdef.GPIO_Speed = GPIO_Speed_10MHz;
	GpioInitdef.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_Init(GPIOX_CLK, &GpioInitdef);
	
	GpioInitdef.GPIO_Pin = GPIOX_PIN_SIMO;
	GpioInitdef.GPIO_Speed = GPIO_Speed_10MHz;
	GpioInitdef.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_Init(GPIOX_SIMO, &GpioInitdef);

	GPIO_SetBits(GPIOX_STE, GPIOX_PIN_STE);
	GPIO_SetBits(GPIOX_CLK, GPIOX_PIN_CLK);
	GPIO_SetBits(GPIOX_WE, GPIOX_PIN_WE);
 }

 static void Delay(vu32 nCount)
 {
	for(; nCount != 0; nCount--);
 }

 INT8U SPIDataXchg( INT8U dt )
 {
 	INT8S i;
	INT8U data = 0;
	
	for( i=7; i>=0; --i )
	{
		GPIO_ResetBits(GPIOX_CLK, GPIOX_PIN_CLK);
		GPIO_WriteBit(GPIOX_SIMO, GPIOX_PIN_SIMO, ((dt&(0x01<<i))!=0)? Bit_SET: Bit_RESET );
		Delay(1);
		GPIO_SetBits(GPIOX_CLK, GPIOX_PIN_CLK);
		data <<= 1;
		data |= ((Bit_SET == GPIO_ReadInputDataBit(GPIOX_SOMI, GPIOX_PIN_SOMI))?1:0);
		Delay(1);
	}
		
	return data;
 }

 
 void SPISend( const INT8U* buf, INT16U len )
 {
 
	 while( len-- > 0 )
	 {
		 SPIDataXchg( *buf++ );
	 }
 }
 
 void SPIReceive( INT8U *buf, INT16U len )
 {
 
	 while( len-- > 0 )
	 {
	 	*buf++ = SPIDataXchg( 0 );
	 }
 }


static void WriteEnable()
{
	INT8U buf[2] = {WREN};
	
	SelectChip();

	SPISend( buf,1 );

	DeSelectChip();
}


static INT16U e2prompagewt( INT16U addr, INT8U *data, INT16U len )
{
	INT16U	len_t;
	INT8U  buf[4];

	buf[0] = WRITE;
	buf[1] = addr>>8;
	buf[2] = addr;
	
	WriteEnable();	
	GPIO_ResetBits(GPIOX_WE, GPIOX_PIN_WE);		//WPE = 0;
#ifdef UCOSIII
	OSTimeDly(	(OS_TICK	)1,
				(OS_OPT		)OS_OPT_TIME_DLY,
				(OS_ERR		*)&err	);
#else
	Delay(1000);
#endif
	SelectChip();

	SPISend( buf,3 );
	len_t = (len>ROM_BLOCK_SIZE)? ROM_BLOCK_SIZE: len;
	SPISend( data, len_t );
	DeSelectChip();
#ifdef UCOSIII
	OSTimeDly(	(OS_TICK	)1,
				(OS_OPT		)OS_OPT_TIME_DLY,
				(OS_ERR		*)&err	);
#else
	Delay(1000);
#endif

	do
	{
		SelectChip();
		buf[0] = RDSR;
		SPISend( buf,1 );
		SPIReceive( buf, 1 );
		DeSelectChip();
#ifdef UCOSIII
		OSTimeDly(	(OS_TICK	)1,
					(OS_OPT		)OS_OPT_TIME_DLY,
					(OS_ERR		*)&err	);
#else
		Delay(1000);
#endif

	}
	while( (buf[0]&0x01) > 0 );

	GPIO_SetBits(GPIOX_WE, GPIOX_PIN_WE);//WPE = 1;
	
	return len_t;
}

void	e2promwt_ext( INT16U addr, INT8U *data, INT16U len )
{
	INT16U	page_addr = addr&~0x1F, lent;
	
	addr = addr-page_addr;
	if( addr )
	{
		lent = (addr>=len)? len: addr;
		lent = e2prompagewt( page_addr+addr, data, lent );
		len -= lent;
		data += addr;
	}
	while( len )
	{
		lent = e2prompagewt( page_addr, data, len );
		len -= lent;
		page_addr += lent;
		data += lent;
	}
}

void	e2promrd_ext( INT16U addr, INT8U *buf, INT16U len )
{
	INT8U snd[4];
	
	SelectChip();
	snd[0] = READ;
	snd[1] = addr>>8;
	snd[2] = addr;

	SPISend( snd, 3 );
	SPIReceive( buf, len );
	
	DeSelectChip();
}



/*******************************************************************************
* Function Name :static INT32S EepromOpen(void *pd)
* Description   :ⲿ洢оƬ25LC640ͨŶ˿ڳʼ
* Input         :
* Output        :
* Other         :
* Date          :2012.05.28  16:17:21
*******************************************************************************/
static INT32S EepromOpen(void *pd)
{
	EepromInit();
	return (INT32S)DRV_NO_ERR;
}


/*******************************************************************************
* Function Name :static INT32S EepromClose(void *pd)
* Description   :Ч
* Input         :
* Output        :
* Other         :
* Date          :2012.05.28  16:18:25
*******************************************************************************/
static INT32S EepromClose(void *pd)
{	
	
	return (INT32S)DRV_NO_ERR;
}


/*******************************************************************************
* Function Name :static INT32S EepromWrite(INT8S *buffer, INT32U lenToWrite, INT8U waitType)
* Description   :дݵⲿ洢оƬ25LC640
* Input         :buffer:ҪдݡlenToWriter:ݳ(ֽ)
* Output        :
* Other         :
* Date          :2012.05.28  16:18:46
*******************************************************************************/
static INT32S EepromWrite(INT8S *buffer, INT32U lenToWrite, INT8U waitType)
{
	e2promwt_ext(0x00, (INT8U *)buffer, lenToWrite);

	return (INT32S)DRV_NO_ERR;
}

/*******************************************************************************
* Function Name :static INT32S EepromRead(INT8S *buffer, INT32U blen, INT32U lenToRead, INT8U waitType)
* Description   :ⲿ洢оƬ25LC640 洢
* Input         :buffer:ݻlenToRead:ݳ(ֽ)
* Output        :
* Other         :
* Date          :2012.05.28  16:19:26
*******************************************************************************/
static INT32S EepromRead(INT8S *buffer, INT32U blen, INT32U lenToRead, INT8U waitType)
{	
	e2promrd_ext(0x00, (INT8U *)buffer, lenToRead);
	
	return (INT32S)DRV_NO_ERR;
}


INT32S EepromInstall(UDFOperationsType *op)
{
	op->devOpen = EepromOpen;
	op->devClose = EepromClose;
	op->devWrite = EepromWrite;
	op->devRead = EepromRead;
	op->devIoctl = NULL;
	
	return (INT32S)DRV_NO_ERR;
}


INT32S EepromNuinstall(UDFOperationsType *op)
{
	INT32S res = (INT32S)DRV_NO_ERR;
	void *pd = NULL;
	
	if (op->devClose != NULL)
		res = op->devClose(pd);
	op->devOpen = NULL;
	op->devClose = NULL;
	op->devWrite = NULL;
	op->devRead = NULL;
	op->devIoctl = NULL;

	return res;
}









