/*
********************************************************************************
*                                  uC/OS-II
*							   Flash
*                              ARM Cortex-M3 Port
*
* File      	: DrvFlash.C
* Version   	: V1.0
* By        	: ǿ
*
* For       	: Stm32f10x
* Mode      	: Thumb2
* Toolchain 	: 
*             		RealView Microcontroller Development Kit (MDK)
*             		Keil uVision
* Description   : STM32F10x ڲFlash 
*					ռFlash
*				
*					FlashOpen
*					FlashClose
*					FlashWrite
*					FlashRead
*					FlashIoCtl
*					FlashInstall
*					FlashNuinstall
* Date          : 2012.06.21
*******************************************************************************/

#include "DrvFlash.h"


/* Private typedef -----------------------------------------------------------*/
typedef enum {FAILED = 0, PASSED = !FAILED} TestStatus;

/* Private define ------------------------------------------------------------*/
/* Define the STM32F10x FLASH Page Size depending on the used STM32 device */
#if defined (STM32F10X_HD) || defined (STM32F10X_HD_VL) || defined (STM32F10X_CL) || defined (STM32F10X_XL)
  #define FLASH_PAGE_SIZE    ((uint16_t)0x800)
#else
  #define FLASH_PAGE_SIZE    ((uint16_t)0x400)
#endif

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/  
volatile FLASH_Status FLASHStatus = FLASH_COMPLETE;
volatile TestStatus MemoryProgramStatus = PASSED;

static vu8 flashOpendFlg = FALSE;

/*******************************************************************************
* Function Name :static INT32S FlashOpen(void *pd)
* Description   :FLASH  Bank1
* Input         :Ч
* Output        :
* Other         :
* Date          :2012.06.21  13:40:57
*******************************************************************************/
static INT32S FlashOpen(void *pd)
{
	flashOpendFlg = TRUE;
	return (INT32S)DRV_NO_ERR;
}

/*******************************************************************************
* Function Name :static INT32S FlashClose(void *pd)
* Description   :FLASH  Bank1
* Input         :Ч
* Output        :
* Other         :
* Date          :2012.06.21
*******************************************************************************/
static INT32S FlashClose(void *pd)
{	
	flashOpendFlg = FALSE;
	return (INT32S)DRV_NO_ERR;
}

/*******************************************************************************
* Function Name :static INT32S FlashWrite(INT8S *buffer, INT32U lenToWrite, INT8U waitType)
* Description   :дFLASH
* Input         :*buffer:塣lenToWrite:ݵĳ(ֽ)
* Output        :
* Other         :
* Date          :2012.06.21
*******************************************************************************/
static INT32S FlashWrite(INT8S *buffer, INT32U lenToWrite, INT8U waitType)
{
	vu32 * volatile data = (u32 *)buffer;
	vu32 Address = BANK1_WRITE_START_ADDR;
	u32 NbrOfPage = 0;
	vu32 EraseCounter = 0;
#ifndef UCOSIII
	OS_ERR err;
#endif	

	if (!flashOpendFlg)		//ûб ֱ˳
	{
		return(INT32S)DRV_WRITE_FAIL;
	}
	//ȳΧ ֱ˳
	if (lenToWrite > BANK1_USE_LEN_ADDR)
	{
		return(INT32S)DRV_WRITE_FAIL;
	}
	
	__disable_irq();				//ж
#ifndef UCOSIII
...	OSSchedLock((OS_ERR *)&err);	//ӵ
#endif
	FLASH_UnlockBank1();		//FLASH
	
	/* Define the number of page to be erased */
	NbrOfPage = BANK1_USE_LEN_ADDR / FLASH_PAGE_SIZE;

	/* Clear All pending flags */
	FLASH_ClearFlag(FLASH_FLAG_EOP | FLASH_FLAG_PGERR | FLASH_FLAG_WRPRTERR); 

	/* Erase the FLASH pages */
	for(EraseCounter = 0; (EraseCounter < NbrOfPage) && (FLASHStatus == FLASH_COMPLETE); EraseCounter++)
	{
		FLASHStatus = FLASH_ErasePage(BANK1_WRITE_START_ADDR + (FLASH_PAGE_SIZE * EraseCounter));
	}

	/* Program Flash Bank1 */
	Address = BANK1_WRITE_START_ADDR;

	while((Address < (BANK1_WRITE_START_ADDR + lenToWrite)) && (FLASHStatus == FLASH_COMPLETE))
	{
		FLASHStatus = FLASH_ProgramWord(Address, *data++);
		Address = Address + 4;
	}
	
	FLASH_LockBank1();
#ifndef UCOSIII
	OSSchedUnlock((OS_ERR *)&err);
#endif	
	__enable_irq();		//ж

	return (INT32S)DRV_NO_ERR;
}

/*******************************************************************************
* Function Name :static INT32S FlashRead(INT8S *buffer, INT32U blen, INT32U lenToRead, INT8U waitType)
* Description   :FLASHд洢
* Input         :*buffer:塣lenToRead:ݵĳ(ֽ)
* Output        :
* Other         :
* Date          :2012.06.21
*******************************************************************************/
static INT32S FlashRead(INT8S *buffer, INT32U blen, INT32U lenToRead, INT8U waitType)
{	
	vu32 Address = BANK1_WRITE_START_ADDR;
	u32 * volatile data = (u32 *)buffer;
	
	if (!flashOpendFlg)		//ûб ֱ˳
	{
		return(INT32S)DRV_WRITE_FAIL;
	}
	//ȳΧ ֱ˳
	if (lenToRead > BANK1_USE_LEN_ADDR)
	{
		return(INT32S)DRV_WRITE_FAIL;
	}
	
	for (;Address < BANK1_WRITE_START_ADDR + lenToRead; Address += 4)
	{
		*data++ = *(__IO u32 *)Address;
	}

	return (INT32S)DRV_NO_ERR;
}


/*******************************************************************************
* Function Name :INT32S FlashInstall(UDFOperationsType *op)
* Description   :װFlash
* Input         :
* Output        :
* Other         :
* Date          :2012.06.21
*******************************************************************************/
INT32S FlashInstall(UDFOperationsType *op)
{
	op->devOpen = FlashOpen;
	op->devClose = FlashClose;
	op->devWrite = FlashWrite;
	op->devRead = FlashRead;
	op->devIoctl = NULL;
	
	return (INT32S)DRV_NO_ERR;
}

/*******************************************************************************
* Function Name :INT32S FlashNuinstall(UDFOperationsType *op)
* Description   :жFlash
* Input         :
* Output        :
* Other         :
* Date          :2012.06.21
*******************************************************************************/
INT32S FlashNuinstall(UDFOperationsType *op)
{
	INT32S res = (INT32S)DRV_NO_ERR;
	void *pd = NULL;
	
	if (op->devClose != NULL)
		res = op->devClose(pd);
	op->devOpen = NULL;
	op->devClose = NULL;
	op->devWrite = NULL;
	op->devRead = NULL;
	op->devIoctl = NULL;

	return res;
}


/********************** END ***************************************************/


