/*
********************************************************************************
*                                  uC/OS-II
*							   I2Cx
*                              ARM Cortex-M3 Port
*
* File      	: DrvI2Cx.C
* Version   	: V1.0
* By        	: ǿ
*
* For       	: Stm32f10x
* Mode      	: Thumb2
* Toolchain 	: 
*             		RealView Microcontroller Development Kit (MDK)
*             		Keil uVision
* Description   : STM32F10x ڲI2Cx 
*					ռI2C1
*				
*					I2CxOpen
*					I2CxClose
*					I2CxWrite
*					I2CxRead
*					I2CxIoCtl
*					I2CxInstall
*					I2CxNuinstall
* Date          : 2012.11.29
*******************************************************************************/

#include "DrvI2C1.h"


static I2CxType I2CxParam       = CLR;
static I2CxType *pI2C           = &I2CxParam;

#ifdef UCOSIII
static OS_ERR err;
static OS_SEM semI2C;
#endif

/*******************************************************************************
* Function Name :static void I2CxInit(void)
* Description   :I2Cʼ
* Input         :
* Output        :
* Other         :
* Date          :2012.07.27  13:55:29
*******************************************************************************/
static void I2CxInit(void)
{	
    GPIO_InitTypeDef GPIO_InitStructure;
    I2C_InitTypeDef  I2C_InitStructure;

    RCC_APB1PeriphClockCmd(RCC_APBXPeriph_I2C, ENABLE);
    RCC_APB2PeriphClockCmd(RCC_APBXPeriph_GPIOX_SCL |RCC_APBXPeriph_GPIOX_SDA, ENABLE);

    GPIO_InitStructure.GPIO_Pin =  GPIOX_PIN_SDA;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_OD;
    GPIO_Init(GPIOX_SDA, &GPIO_InitStructure);

    GPIO_InitStructure.GPIO_Pin =  GPIOX_PIN_SCL;
    GPIO_Init(GPIOX_SCL, &GPIO_InitStructure);

    I2C_DeInit(I2Cx);
    /* I2Cx configuration ------------------------------------------------------*/
    I2C_InitStructure.I2C_Mode = I2C_Mode_I2C;
    I2C_InitStructure.I2C_DutyCycle = I2C_DutyCycle_2;
    I2C_InitStructure.I2C_OwnAddress1 = 0x30;
    I2C_InitStructure.I2C_Ack = I2C_Ack_Enable;
    I2C_InitStructure.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;
    I2C_InitStructure.I2C_ClockSpeed = ClockSpeed;
    I2C_Init(I2Cx, &I2C_InitStructure);

    I2C_Cmd(I2Cx, ENABLE);	
    I2C_AcknowledgeConfig(I2Cx, ENABLE);			  //ʹI2CӿӦ

}

/*******************************************************************************
* Function Name :static void I2CWriteByte (INT8U addr, INT8U data)
* Description   :дһֽ
* Input         :addr:Ĵַdata:
* Output        :
* Other         :
* Date          :2012.07.27  13:55:46
*******************************************************************************/
static void I2CWriteByte (INT8U addr, INT8U data)
{
    INT16U i = 0;

    memset(pI2C, 0, sizeof(I2CxType));		//
    pI2C->buf[0] = addr;
    pI2C->buf[1] = data;
    pI2C->len = 2;
    pI2C->r_w = WR;

    I2C_ITConfig(I2Cx, I2C_IT_EVT | I2C_IT_BUF, ENABLE);
    I2C_AcknowledgeConfig(I2Cx, ENABLE);
    /* Send I2Cx START condition */
    I2C_GenerateSTART(I2Cx, ENABLE);
#ifdef UCOSIII
    API_UNLOCK();                           //,񱻹ǰͷȨ
    OSSemPend(      (OS_SEM 	*)&semI2C, 
                    (OS_TICK 	)10, 
                    (OS_OPT		)OS_OPT_PEND_BLOCKING, 
                    (CPU_TS		*)0, 
                    (OS_ERR 	*)&err);
    API_LOCK();
#endif
    while (!(pI2C->wFlag))ENDLESS_LOOP_BREAK(i);
    I2C_GenerateSTOP(I2Cx, ENABLE);
    I2C_ITConfig(I2Cx, I2C_IT_EVT | I2C_IT_BUF, DISABLE);
    I2C_AcknowledgeConfig(I2Cx, DISABLE);

}

/*******************************************************************************
* Function Name :static INT8U I2CReadByte (INT8U addr)
* Description   :һֽ
* Input         :addr:Ĵַ
* Output        :
* Other         :
* Date          :2012.07.27  13:55:57
*******************************************************************************/
static INT8U I2CReadByte (INT8U addr)
{
	INT16U i = 0;

	memset(pI2C, 0, sizeof(*pI2C));		//
	pI2C->buf[0] = addr;
	pI2C->len = 2;
	pI2C->r_w = RD;
	
	I2C_ITConfig(I2Cx, I2C_IT_EVT | I2C_IT_BUF, ENABLE);
	I2C_AcknowledgeConfig(I2Cx, ENABLE);
	/* Send I2Cx START condition */
	I2C_GenerateSTART(I2Cx, ENABLE);
#ifdef UCOSIII
    API_UNLOCK();                       //,񱻹ǰͷȨ
	OSSemPend(  (OS_SEM     *)&semI2C, 
				(OS_TICK    )10, 
				(OS_OPT	    )OS_OPT_PEND_BLOCKING, 
				(CPU_TS     *)0, 
				(OS_ERR     *)&err);
    API_LOCK();
#endif
	while (!(pI2C->rFlag))ENDLESS_LOOP_BREAK(i);
	
	I2C_AcknowledgeConfig(I2Cx, DISABLE);
	
	return pI2C->buf[1];
}

/*******************************************************************************
* Function Name :void I2CReadBytes(INT16U *buf, INT16U len, INT8U addr)
* Description   :addrΪ׵ַ lenݡ
* Input         :
* Output        :
* Other         :
* Date          :2012.07.27  15:16:49
*******************************************************************************/
void I2CReadBytes(INT16U *buf, INT16U len, INT8U addr)
{
	INT16U i = 0;
	for (i = 0; i < len / 2; i++)
	{
		*buf = I2CReadByte(addr++) << 8;
		*buf++ |= I2CReadByte(addr++);
	}
}

/*******************************************************************************
* Function Name :void I2CWriteBytes(INT16U *buf, INT16U len, INT8U addr)
* Description   :дһ
* Input         :
* Output        :
* Other         :
* Date          :2012.07.27  15:16:49
*******************************************************************************/
void I2CWriteBytes(INT16U *buf, INT16U len, INT8U addr)
{
	INT16U i = 0;
	for (i = 0; i < len / 2; i++)
	{
		I2CWriteByte(addr++, (*buf >> 8));
		I2CWriteByte(addr++, (*buf++ & 0xFF));
	}
}


/*******************************************************************************
* Function Name :void SCCBInit(void)
* Description   :оƬ
* Input         :
* Output        :
* Other         :
* Date          :2012.07.27  13:56:52
*******************************************************************************/
ErrorStatus SCCBInit(void)
{
	INT32S sum[3] = {0,};
	INT16S offset[3] = {0,};
	INT16U i = 0;
	INT16U j = 0;
	
	I2CxInit();
	I2CxPrioritySet(0x0000);
	
	return SUCCESS;
}

/*******************************************************************************
* Function Name :static void I2Cx(INT16U level)
* Description   :XXXжϵȼ趨
* Input         :level:8λ Ϊռȼ8λǴȼ
* Output        :
* Other         :
* Date          :2012.05.28  10:09:37
*******************************************************************************/
static void I2CxPrioritySet(INT16U level)
{
	NVIC_InitTypeDef NvicInitdef;

	NvicInitdef.NVIC_IRQChannel = I2Cx_EV_IRQn;
	NvicInitdef.NVIC_IRQChannelPreemptionPriority = level >> 8;
	NvicInitdef.NVIC_IRQChannelSubPriority = level & 0xFF;
	NvicInitdef.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init( &NvicInitdef );
}


/*******************************************************************************
* Function Name :static INT32S I2CxOpen(void *pd)
* Description   :оƬʼ
* Input         :pd:NULL
* Output        :
* Other         :
* Date          :2012.07.27  15:37:41
*******************************************************************************/
static INT32S I2CxOpen(void *pd)
{
	if (SCCBInit() == SUCCESS)
		return (INT32S)DRV_NO_ERR;
	else
		return (INT32S)DRV_READ_FAIL;
}

/*******************************************************************************
* Function Name :static INT32S I2CxClose(void *pd)
* Description   :оƬر
* Input         :pd:NULL
* Output        :
* Other         :
* Date          :2012.07.27  15:37:43
*******************************************************************************/
static INT32S I2CxClose(void *pd)
{	
	I2C_Cmd(I2Cx, DISABLE);
	return (INT32S)DRV_NO_ERR;
}

/*******************************************************************************
* Function Name :static INT32S I2CxWrite(INT8S *buffer, INT32U lenToWrite, INT8U type)
* Description   :дݵоƬĴ
* Input         :
* Output        :
* Other         :
* Date          :2012.07.27  15:37:45
*******************************************************************************/
static INT32S I2CxWrite(INT8S *buffer, INT32U lenToWrite, INT8U type)
{
	return (INT32S)DRV_NO_ERR;
}

/*******************************************************************************
* Function Name :INT32S I2CxRead
* Description   :mpu3050Ĵ
* Input         :type
* Output        :
* Other         :
* Date          :2012.07.27  15:27:53
*******************************************************************************/
static INT32S I2CxRead(INT8S *buffer, INT32U blen, INT32U lenToRead, INT8U type)
{	
	switch (type)
	{
		default : break;
	}
	return (INT32S)DRV_NO_ERR;
}

/*******************************************************************************
* Function Name :static INT32S I2CxIoCtl(INT32U too, void *pd)
* Description   :
* Input         :too: 	
*				 DEV_CTRL_NVIC_Priority:	жȼ8λռȼ8λȼ
* Output        :
* Other         :
* Date          :2012.05.23  14:19:51
*******************************************************************************/
static INT32S I2CxIoCtl(INT32U too, void *pd)
{
	switch (too)
	{
	    case DEV_CTRL_NVIC_Priority:
	        break;
	        
		default : 
		    break;
	}
	
	return (INT32S)DRV_NO_ERR;
}
/*******************************************************************************
* Function Name :INT32S I2CxInstall(UDFOperationsType *op)
* Description   :װI2Cx
* Input         :
* Output        :
* Other         :
* Date          :2012.05.13
*******************************************************************************/
INT32S I2CxInstall(UDFOperationsType *op)
{
	op->devOpen = I2CxOpen   ;
	op->devClose = I2CxClose;
	op->devWrite = NULL;
	op->devRead = I2CxRead;
	op->devIoctl = I2CxIoCtl;
#ifdef UCOSIII
	OSSemCreate(    (OS_SEM 		*)&semI2C, 
					(CPU_CHAR 		*)"i2cݽշɱ־", 
					(OS_SEM_CTR	 )0, 
					(OS_ERR 		*)&err);
#endif	
	return DRV_NO_ERR;
}

/*******************************************************************************
* Function Name :INT32S I2CxNuinstall(UDFOperationsType *op)
* Description   :жI2Cx
* Input         :
* Output        :
* Other         :
* Date          :2012.05.13
*******************************************************************************/
INT32S I2CxNuinstall(UDFOperationsType *op)
{
	INT32S res = (INT32S)DRV_NO_ERR;
	void *pd = NULL;
	
	if (op->devClose != NULL)
		res = op->devClose(pd);
	op->devOpen = NULL;
	op->devClose = NULL;
	op->devWrite = NULL;
	op->devRead = NULL;
	op->devIoctl = NULL;

	return res;
}


/*******************************************************************************
* Function Name  : I2Cx_EV_IRQHandler
* Description    : This function handles I2Cx Event interrupt request.
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void I2Cx_EV_IRQHandler(void)
{
#ifdef UCOSIII
    OSIntEnter();
#endif
    switch (I2C_GetLastEvent(I2Cx))
    {
    /* Test on I2Cx EV5 and clear it */
    case I2C_EVENT_MASTER_MODE_SELECT:  
        if ((pI2C->r_w == RD) && (pI2C->index == 1))
        {
            /* Send I2Cx slave Address for write */
            I2C_Send7bitAddress(I2Cx, I2Cx_SLAVE_ADDRESS7, I2C_Direction_Receiver);
        }
        else
        {
            I2C_Send7bitAddress(I2Cx, I2Cx_SLAVE_ADDRESS7, I2C_Direction_Transmitter);
        }
        break;

    /* Test on I2Cx EV6 and first EV8 and clear them */
    case I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED:
        /* Send the first data */
        I2C_SendData(I2Cx, pI2C->buf[pI2C->index++]);  /* EV8 just after EV6 */
        break;

    case I2C_EVENT_MASTER_BYTE_TRANSMITTING:
        if((pI2C->index < pI2C->len)&&(pI2C->r_w == WR))
        {
            /* Transmit buffer data */
            I2C_SendData(I2Cx, pI2C->buf[pI2C->index++]);
        }
        else
        {	
            I2C_ITConfig(I2Cx, I2C_IT_BUF, DISABLE);
        }
        break;

    /* Test on I2Cx EV8 and clear it */
    case I2C_EVENT_MASTER_BYTE_TRANSMITTED:     
        if (pI2C->r_w == RD)
        {
            I2C_ITConfig(I2Cx, I2C_IT_BUF, ENABLE);
            I2C_GenerateSTART(I2Cx, ENABLE);
        }
        else 
        {
            I2C_GenerateSTOP(I2Cx, ENABLE);
            I2C_ITConfig(I2Cx, I2C_IT_EVT, DISABLE);
            pI2C->wFlag = 1;		//ս־
#ifdef UCOSIII
            OSSemPost(  (OS_SEM     *)&semI2C, 
                        (OS_OPT     )OS_OPT_POST_1, 
                        (OS_ERR     *)&err);
#endif
        }
        break;

    /* Test on I2Cx EV6 and first EV8 and clear them */
    case I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED: 
        if (pI2C->index == pI2C->len - 1)
        {
            I2C_AcknowledgeConfig(I2Cx, DISABLE);
            I2C_GenerateSTOP(I2Cx, ENABLE);
        }
        break;

    /* Test on I2Cx EV2 and clear it */
    case I2C_EVENT_MASTER_BYTE_RECEIVED:
        pI2C->buf[pI2C->index++] = I2C_ReceiveData(I2Cx);

        if (pI2C->index == pI2C->len - 1)
        {
            I2C_AcknowledgeConfig(I2Cx, DISABLE);
            I2C_GenerateSTOP(I2Cx, ENABLE);
        }
        else if (pI2C->index >= pI2C->len)
        {
            pI2C->rFlag = 1;
#ifdef UCOSIII
            OSSemPost(  (OS_SEM     *)&semI2C, 
                        (OS_OPT     )OS_OPT_POST_1, 
                        (OS_ERR     *)&err);
#endif
        }
        break; 

    default:
        break;
    }
#ifdef UCOSIII
    OSIntExit();
#endif
}


/********************** END ***************************************************/

